import { FetchCache, CacheState } from './cache';
import { FetchAction } from './action';
import mitt from 'mitt';
export declare type FetchClientDispatcher = (action: FetchAction<any>) => Promise<{
    status?: number;
    error?: Error;
    data?: any;
    headers?: Headers;
}>;
export declare const defaultDispatcher: FetchClientDispatcher;
export interface IdFromActionFn {
    (action: FetchAction<any>): string;
}
export interface ShouldCacheActionFn {
    (action: FetchAction<any>): boolean;
}
interface EventsMap {
    update?: (newValue: any) => void;
    revalidate?: () => void;
}
export declare class FetchClient {
    cache: FetchCache;
    idFromAction: IdFromActionFn;
    shouldCacheAction: ShouldCacheActionFn;
    queue: Promise<any>[];
    protected queueMap: {
        [key in string]: Promise<any>;
    };
    protected emitter: mitt.Emitter;
    protected dispatcher: FetchClientDispatcher;
    constructor(cache: FetchCache, dispatcher?: FetchClientDispatcher, idFromAction?: IdFromActionFn, shouldCacheAction?: ShouldCacheActionFn);
    dispatch(action: FetchAction<any>): Promise<any>;
    listen(actionId: string, eventsMap: EventsMap): () => void;
    mutate(actionId: string, newValue: any): void;
    refetchQueries(): void;
    waitForQueue(): Promise<any[]>;
}
interface CreateClientConfig {
    cache?: FetchCache;
    initialCacheState?: CacheState;
    dispatcher?: FetchClientDispatcher;
}
export declare const createClient: ({ cache, initialCacheState, dispatcher, }: CreateClientConfig) => FetchClient;
export {};
