"use strict";

exports.__esModule = true;
exports.createClient = exports.FetchClient = exports.defaultDispatcher = void 0;

var _cache = require("./cache");

var _mitt = _interopRequireDefault(require("mitt"));

var _isomorphicUnfetch = _interopRequireDefault(require("isomorphic-unfetch"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }

function _toPropertyKey(arg) { var key = _toPrimitive(arg, "string"); return typeof key === "symbol" ? key : String(key); }

function _toPrimitive(input, hint) { if (typeof input !== "object" || input === null) return input; var prim = input[Symbol.toPrimitive]; if (prim !== undefined) { var res = prim.call(input, hint || "default"); if (typeof res !== "object") return res; throw new TypeError("@@toPrimitive must return a primitive value."); } return (hint === "string" ? String : Number)(input); }

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

var defaultDispatcher = function defaultDispatcher(action) {
  return (0, _isomorphicUnfetch["default"])(action.url, action).then(function (resp) {
    return resp.json().then(function (data) {
      return {
        data: data,
        status: resp.status,
        headers: resp.headers
      };
    })["catch"](function (error) {
      return {
        error: error,
        status: resp.status,
        headers: resp.headers
      };
    });
  })["catch"](function (error) {
    return {
      error: error,
      status: error.status,
      headers: error.headers
    };
  });
};

exports.defaultDispatcher = defaultDispatcher;

var defaultIdFromAction = function defaultIdFromAction(action) {
  return action.id || action.url;
};

var defaultShouldCacheAction = function defaultShouldCacheAction(action) {
  return action.method === 'GET';
};

var FetchClient =
/*#__PURE__*/
function () {
  function FetchClient(cache, dispatcher, idFromAction, shouldCacheAction) {
    this.cache = cache;
    this.dispatcher = dispatcher || defaultDispatcher;
    this.idFromAction = idFromAction || defaultIdFromAction;
    this.shouldCacheAction = shouldCacheAction || defaultShouldCacheAction;
    this.queue = [];
    this.queueMap = {};
    this.emitter = (0, _mitt["default"])();
  }

  var _proto = FetchClient.prototype;

  _proto.dispatch = function dispatch(action) {
    var _this = this;

    var actionId = action.id || this.idFromAction(action); // Return promise in queue if it exists

    if (this.queueMap[actionId]) {
      return this.queueMap[actionId];
    }

    var dispatchedAction = _extends({}, action, {
      id: actionId
    });

    var promise = this.dispatcher(dispatchedAction);
    this.queue.push(promise);
    this.queueMap[actionId] = promise;
    return promise.then(function (value) {
      _this.queue = _this.queue.filter(function (p) {
        return p !== promise;
      });

      var _this$queueMap = _this.queueMap,
          deleted = _this$queueMap[actionId],
          nextQueueMap = _objectWithoutPropertiesLoose(_this$queueMap, [actionId].map(_toPropertyKey));

      _this.queueMap = nextQueueMap;

      var shouldCache = _this.shouldCacheAction(action);

      if (shouldCache) {
        _this.cache.set(dispatchedAction, value);

        _this.emitter.emit('update', {
          actionId: actionId,
          data: value
        });
      }

      return value;
    });
  };

  _proto.listen = function listen(actionId, eventsMap) {
    var _this2 = this;

    var handler = function handler(eventName, event) {
      if (eventName && event.actionId === actionId && eventsMap[eventName]) {
        eventsMap[eventName](event.data);
      }
    };

    this.emitter.on('*', handler);
    return function () {
      return _this2.emitter.off('*', handler);
    };
  };

  _proto.mutate = function mutate(actionId, newValue) {
    this.cache.updateValue(actionId, newValue);
    this.emitter.emit('update', {
      actionId: actionId,
      data: newValue
    });
  };

  _proto.refetchQueries = function refetchQueries() {};

  _proto.waitForQueue = function waitForQueue() {
    if (!this.queue) {
      throw new Error('Queue is only used when ssr is enabled');
    }

    return Promise.all(this.queue);
  };

  return FetchClient;
}();

exports.FetchClient = FetchClient;

var createClient = function createClient(_ref) {
  var cache = _ref.cache,
      initialCacheState = _ref.initialCacheState,
      dispatcher = _ref.dispatcher;
  return new FetchClient(cache || (0, _cache.createCache)({
    initialState: initialCacheState
  }), dispatcher);
};

exports.createClient = createClient;