"use strict";

exports.__esModule = true;
exports["default"] = useFetch;

var _react = require("react");

var _context = require("./context");

var _actHack = require("./act-hack");

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

/**
 *
 * @param uri URI to be fetched from
 * @param options It's good practice to memoize this param
 */
var IS_SERVER = typeof window === 'undefined';
var useIsomorphicLayoutEffect = IS_SERVER ? _react.useEffect : _react.useLayoutEffect;

function useFetch(action) {
  var client = (0, _react.useContext)(_context.FetchContext);

  if (!client) {
    throw new Error('useFetch should not be used outside <FetchClientProvider />');
  }

  var actionId = action && client.idFromAction(action);
  var cachedValue = actionId ? client.cache.get(actionId) : undefined;

  var _useState = (0, _react.useState)(_extends({}, cachedValue, {
    action: action || (cachedValue === null || cachedValue === void 0 ? void 0 : cachedValue.action) || undefined,
    loading: !cachedValue
  })),
      state = _useState[0],
      setState = _useState[1];

  (0, _react.useEffect)(function () {
    if (cachedValue && (state.data !== cachedValue.data || state.status !== cachedValue.status || state.action !== cachedValue.action)) {
      setState(_extends({}, cachedValue, {
        action: action || undefined,
        loading: false
      }));
    }
  }, [cachedValue, actionId]);
  (0, _react.useMemo)(function () {
    if (actionId) {
      var unlisten = client.listen(actionId, {
        update: function update(newValue) {
          setState(newValue);
        },
        revalidate: function revalidate() {}
      });

      if (action && !cachedValue) {
        if (state.loading !== true) {
          setState({
            action: action,
            loading: true
          });
        }

        client.dispatch(action)["catch"](function (error) {
          (0, _actHack.actHack)(function () {
            setState({
              error: error,
              action: action,
              loading: false
            });
          });
        }).then(function (result) {
          (0, _actHack.actHack)(function () {
            setState(_extends({}, result, {
              action: action,
              loading: false
            }));
          });
        });
      }

      return unlisten;
    }
  }, [client, actionId]);
  return state;
}